@@ -3,7 +3,7 @@
 '''
 A library to access Tryton's models like a client.
 '''
-__version__ = "2.0.0"
+__version__ = "2.2.1"
 __all__ = ['Model', 'Wizard']
 from types import NoneType
 import threading
@@ -55,6 +55,7 @@ class FieldDescriptor(object):
         super(FieldDescriptor, self).__init__()
         self.name = name
         self.definition = definition
+        self.__doc__ = definition['string']
 
     def __get__(self, instance, owner):
         if instance.id > 0 and self.name not in instance._values:
@@ -87,7 +88,15 @@ class CharDescriptor(FieldDescriptor):
 
     def __set__(self, instance, value):
         assert isinstance(value, basestring) or value in (None, False)
-        super(CharDescriptor, self).__set__(instance, value)
+        super(CharDescriptor, self).__set__(instance, value or '')
+
+
+class BinaryDescriptor(FieldDescriptor):
+    default = None
+
+    def __set__(self, instance, value):
+        assert isinstance(value, (basestring, buffer)) or value in (None, False)
+        super(BinaryDescriptor, self).__set__(instance, value or '')
 
 
 class IntegerDescriptor(FieldDescriptor):
@@ -173,6 +182,10 @@ class Many2OneDescriptor(FieldDescriptor):
         super(Many2OneDescriptor, self).__set__(instance, value)
 
 
+class One2OneDescriptor(Many2OneDescriptor):
+    pass
+
+
 class One2ManyDescriptor(FieldDescriptor):
     default = []
 
@@ -218,6 +231,10 @@ class Many2OneValueDescriptor(ValueDescriptor):
         return value and value.id or False
 
 
+class One2OneValueDescriptor(Many2OneValueDescriptor):
+    pass
+
+
 class One2ManyValueDescriptor(ValueDescriptor):
     def __get__(self, instance, owner):
         value = [('add', [])]
@@ -229,8 +246,7 @@ class One2ManyValueDescriptor(ValueDescriptor):
                         fields=record._changed)))
                 value[0][1].append(record.id)
             else:
-                value.append(('create', record._get_values(
-                    fields=record._changed)))
+                value.append(('create', record._get_values()))
         if value_list.record_removed:
             value.append(('unlink', [x.id for x in value_list.record_removed]))
         if value_list.record_deleted:
@@ -268,6 +284,10 @@ class Many2OneEvalDescriptor(EvalDescriptor):
         return False
 
 
+class One2OneEvalDescriptor(Many2OneEvalDescriptor):
+    pass
+
+
 class One2ManyEvalDescriptor(EvalDescriptor):
     def __get__(self, instance, owner):
         return [x.id for x in getattr(instance, self.name)]
@@ -283,7 +303,7 @@ class MetaModelFactory(object):
         'char': CharDescriptor,
         'text': CharDescriptor,
         'sha': CharDescriptor,
-        'binary': CharDescriptor,
+        'binary': BinaryDescriptor,
         'selection': CharDescriptor, # TODO implement its own descriptor
         'integer': IntegerDescriptor,
         'biginteger': IntegerDescriptor,
@@ -296,18 +316,21 @@ class MetaModelFactory(object):
         'many2one': Many2OneDescriptor,
         'one2many': One2ManyDescriptor,
         'many2many': Many2ManyDescriptor,
+        'one2one': One2OneDescriptor,
     }
     value_descriptors = {
         'reference': ReferenceValueDescriptor,
         'many2one': Many2OneValueDescriptor,
         'one2many': One2ManyValueDescriptor,
         'many2many': Many2ManyValueDescriptor,
+        'one2one': One2OneValueDescriptor,
     }
     eval_descriptors = {
         'reference': ReferenceEvalDescriptor,
         'many2one': Many2OneEvalDescriptor,
         'one2many': One2ManyEvalDescriptor,
         'many2many': Many2ManyEvalDescriptor,
+        'one2one': One2OneEvalDescriptor,
     }
 
     def __init__(self, model_name, config=None):
@@ -686,8 +709,12 @@ class Model(object):
                 for vals in value.get('add', []):
                     relation = Model.get(self._fields[field]['relation'],
                             self._config)
+                    record = relation()
+                    for i, j in vals.iteritems():
+                        record._values[i] = j
+                        record._changed.add(i)
                     # append without signal
-                    list.append(getattr(self, field), relation(**vals))
+                    list.append(getattr(self, field), record)
                 for vals in value.get('update', []):
                     if 'id' not in vals:
                         continue
@@ -720,8 +747,6 @@ class Model(object):
                 self._on_change_set(field, value)
             for field, value in later.iteritems():
                 self._on_change_set(field, value)
-            if self._parent:
-                self._parent._changed.add(self._parent_field_name)
         for field, definition in self._fields.iteritems():
             if not definition.get('on_change_with'):
                 continue
@@ -734,6 +759,9 @@ class Model(object):
             res = getattr(self._proxy, 'on_change_with_%s' % field)(args,
                     context)
             self._on_change_set(field, res)
+        if self._parent:
+            self._parent._changed.add(self._parent_field_name)
+            self._parent._on_change(self._parent_field_name)
 
 
 class Wizard(object):
